	subroutine ceq_ck( ns, ne, Ein, thermo, iwk, rwk )

!  Utility routine to obtain the element array Ein and the thermo
!  data from Chemkin (which must have been initialized).

! Input:
!   ns - number of species
!   ne - number of elements
!   iwk - Chemkin integer work array
!   rwk - Chemkin real work array

! Output:
!   Ein    - element matrix (ns x ne).  A molecule of species k contains
!            Ein(k,j) atoms of element j.
!   thermo - thermodynamic data for species (ns x 15)

	implicit none
	integer, intent(in)           :: ns, ne, iwk(*)
	real(kind(1.d0)), intent(in)  :: rwk(*)
	real(kind(1.d0)), intent(out) :: Ein(ns,ne), thermo(ns,15)

	integer :: is, ie, evt(ne,ns)

	call ckncf( ne, iwk, rwk, evt )

	do is = 1, ns
	do ie = 1, ne
	   Ein(is,ie) = evt(ie,is)
	enddo
	enddo

	call ck_thermo(ns, iwk, rwk, thermo)

	return
	end subroutine ceq_ck

!------------------------------------------------
	subroutine ck_thermo(ns, ICKWRK, RCKWRK, thermo)

!  output, on ck_thermo.txt, the thermo data, one line for each species.
!  the data are:  switch temperature; 7 coeffs in lower range; 7 coeffs in upper range

      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)

      DIMENSION ICKWRK(*), RCKWRK(*), thermo(ns,15)
      INCLUDE 'ckstrt.h'

!  loop over species
      DO 250 K = 1, NKK
         L = 1
         DO 220 N = 2, ICKWRK(IcNT + K - 1)-1
            TEMP = RCKWRK(NcTT + (K-1)*MXTP + N - 1)
 220     CONTINUE
	
         NA1 = NcAA + (L-1)*NCP2 + (K-1)*NCP2T
	     NA2 = NcAA +  L*   NCP2 + (K-1)*NCP2T
         
		 thermo(K,1) = TEMP

		 do n=1,ncp2
		    thermo(K,n+1)      = RCKWRK(NA1 + N - 1)
		    thermo(K,n+ncp2+1) = RCKWRK(NA2 + N - 1)
		 enddo

250   CONTINUE
      RETURN
      END subroutine ck_thermo